==> GeneticsBase_1.0.0.tar.gz: GeneticsBase/R/alleleSummary.R <==
## library(gdata)

alleleSummary <- function(object,
                          by=NULL,
                          confidence=0.95,
                          alpha=1-confidence,
                          show=TRUE,
                          verbose=FALSE,
                          includeOverall=FALSE,
                          omitRepeats=TRUE,
                          founderOnly=TRUE,
                          ...)
  {
    if(founderOnly==TRUE)
    { object<-founderGeneSet(object) } 

    ## Ensure that all "Gene" names (if present) appear together in the markerInfo
    if(!is.null(markerInfo(object)$Gene))
      tGene <- markerInfo(object)$Gene
    else
      tGene <- rep(1, nrow(markerInfo(object)))

    if(!is.null(markerInfo(object)$Position))
      tPosition <- markerInfo(object)$Position
    else
      tPosition <- rep(1, nrow(markerInfo(object)))

    if(!is.null(markerInfo(object)$Name))
      tMarker <- markerInfo(object)$Name
    else
      tMarker <- rep(1, nrow(markerInfo(object)))

    ord <- order(tGene, tPosition, tMarker)
    markers <- markerInfo(object)$Name[ord]

    ## Ensure that if by is present then it matches one of the columns of sampleInfo
    if(!missing(by) && !(by %in% colnames(sampleInfo(object))))
      {
        paste("Warning! - by is not present in sampleInfo of the object",
              "Using entire dataset\n")
        by <- "ALL"
      }

    ## If the "by" info is not present in markerInfo, assign it to "ALL"
    if(missing(by))
      by <- "ALL"

    ## Saving sampleInfo as it is used in several places.
    samp.info <- sampleInfo(object)

    ##
    if(!is.null( markerInfo(object)$Gene ) )
       {
         geneList <- sort(unique(as.character( markerInfo(object)$Gene )))
         markerList <- split(markerInfo(object)$Name, markerInfo(object)$Gene)
       }
    else
      {
        geneList <- "ALL"
        markerList <- split(markerInfo(object)$Name, "ALL")
        ## markerList <- as.list(markerInfo(object)$Name)
        ## names(markerList) <- rep("ALL",length(markerList))
      }

    if(!missing(by) && (by %in% colnames(sampleInfo(object))))
      {
        groupList <- levels(as.factor(sampleInfo(object)[,by]))
        if(includeOverall)
          groupList = c("ALL",groupList)
      }
    else
      groupList <- "ALL"

    nmarkers <- nmarker(object)
    nLevels <- length(groupList)

    ##    totalNoAlleles <- sum(nallele(object)) + nmarker(object)
    ##    ## as we want to report the NA summary also


    ## inner calculation from markerLoop here
    totalNoAlleles <- 0
    for (marker in markers)
      {
        aC <- alleleCount(object, marker=marker)
        count <-  apply(aC, 2, sum, na.rm=TRUE)
        names(count) <- unlist(alleleLevels(object, marker))
        count.na <- c(count, "NA"=sum(is.na(aC[,1])))
        totalNoAlleles <- totalNoAlleles + length(count.na)
      }
    
    columnNames <- c("Gene", "Marker", "Position", "Group", "Allele", 
                     "Count", "Freq", "CI-Lower", "CI-Upper")

    len.cols <- length(columnNames)
    resultTab <- matrix("", nrow=(totalNoAlleles + nmarkers)* nLevels   ,
                      ncol=len.cols)  # the +1*.. is for blank separator lines
    colnames(resultTab) <- columnNames

    if(by != "ALL")
    {
      factor.split <- as.factor(samp.info[,by])
      subGroups <- split(samp.info, factor.split)
    }
    else
      {
        subGroups <- split(samp.info, 1)
        names(subGroups) <- "ALL"
      }
        
    counter <- 1
    for(gene in geneList)
      {
        # add gene infomration to the table
        resultTab[counter, "Gene"] <- gene

        markers.gene <- markerList[[gene]]
        ## Marker Loop
        for(marker in markers.gene)
          {
            ##            if (by == "ALL")
            ##            object <- object
            ##        else {
            ##        object <- object[, as.numeric(rownames)]
            ##            }

            if(!omitRepeats)
              resultTab[counter, "Gene"] <- gene
            resultTab[counter, "Marker"] <- marker

            if(!is.null(markerInfo(object)$Position))
              resultTab[counter, "Position"] <-
                as.character(markerInfo(object)[marker,"Position"])
            else
              resultTab[counter, "Position"] <- "?"
            ## Inner most loop is starting here
            for(group in groupList)
              {
                resultTab[counter, "Group"] <- as.vector(groupList[group])

                ## inner calculation from markerLoop here
                ##aC <- alleleCount(object, marker=marker)

                curr.group <- group
                groupName <- group

                if(group=="ALL")
                  {
                    object.curr <- object
                  }
                else
                  {
                    subset.rows <- rownames(subGroups[[curr.group]]) %in% 
                                   rownames(sampleInfo(object))
                    object.curr <- object[,subset.rows]
                  }


                
                aC <- alleleCount(object.curr, marker=marker)
                count <-  apply(aC, 2, sum, na.rm=TRUE)
                names(count) <- unlist(alleleLevels(object.curr, marker))
                count.na <- c(count, "NA"=sum(is.na(aC[,1])))

                freq  <- prop.table(count)
                n <- sum(count)
                ## FIXME: These calculations are based on binomial
                ## probabilities and may not be correct for the true
                ## multinomial distributions
                stderr <- sqrt(freq*(1-freq)/n)
                ci.low  <- qbinom(p=alpha/2, prob=freq, size=n)/n
                ci.high <- qbinom(p=1-alpha/2, prob=freq, size=n)/n

                count.na <- c(count.na, "") ## To incroporate the blank line
                len.count.na <- length(count.na)
                
                resultTab[(counter: (counter+len.count.na-1)), "Allele"] <- sprintf("%s",names(count.na))
                resultTab[(counter: (counter+len.count.na-1)), "Count"] <- sprintf("%6s",count.na)
                resultTab[(counter: (counter+length(freq)-1)), "Freq"] <- sprintf("%1.3f",freq)
                resultTab[(counter: (counter+length(ci.low)-1)), "CI-Lower"] <- sprintf("%1.3f",ci.low)
                resultTab[(counter: (counter+length(ci.high)-1)), "CI-Upper"] <- sprintf("%1.3f",ci.high)


                if(!missing(groupName) && groupName !="ALL")
                  resultTab[(counter: (counter+len.count.na-2)), "Group"] <- groupName
                else
                  resultTab[(counter: (counter+len.count.na-2)), "Group"] <- "ALL"

                if(verbose && counter%%50==0)
                  cat("Working on marker #", counter, "\n",sep="")
                
                counter <- counter+len.count.na
              }
          }
        
        }

    retval <- list()

    rownames(resultTab) <- rep("", nrow(resultTab))
    retval$data <- resultTab
    
    retval$footer <- paste("Confidence intervals width is 95%, computed using",
                           "the exact quantiles for the binomial distribution.")
    
    retval$call <- match.call()

    class(retval) <- c("alleleSummary","GeneticsBaseSummary")
    
    return(retval)
  }



markerLoop <- function(object,
                       groupName,
                       confidence=0.95,
                       alpha=1-confidence,
                       show=TRUE,
                       verbose=FALSE,
                       markers=markerNames(object),
                       ...)
  {

    ## Table columns: 
    ## One line per marker per allele per covariate
    columnNames <- c("Gene", "Marker", "Position", "Group", "Allele", 
                  "Count", "Freq", "CI-Lower", "CI-Upper")


    tableList <- vector(mode="list", length=nmarker(object))
    names(tableList) <- markerNames(object)

    ## Replacing this loop with vector operations (ie for stderr,
    ## ci.low, ci.high, ...) might speed it up considerably
    counter <- 1
    for(marker in markers)
      {
        if(verbose && counter%%50==0)
          cat("Working on marker #", counter, "\n",sep="")
        counter <- counter+1

        aC <- alleleCount(object, marker=marker)

        count <- apply(
                       aC,
                       2,
                       sum,
                       na.rm=TRUE)
        names(count) <- unlist(alleleLevels(object, marker))
        
        count.na <- c(count, "NA"=sum(is.na(aC[,1])))
        
        freq  <- prop.table(count)
        n <- sum(count)
        
        ## FIXME: These calculations are based on binomial
        ## probabilities and may not be correct for the true
        ## multinomial distributions
        stderr <- sqrt(freq*(1-freq)/n)
        ci.low  <- qbinom(p=alpha/2, prob=freq, size=n)/n
        ci.high <- qbinom(p=1-alpha/2, prob=freq, size=n)/n
        
        retval <- matrix("", ncol=length(columnNames), nrow=length(count.na)+1)
        rownames(retval) <- c(names(count.na),"")
        colnames(retval) <- columnNames

        if(!is.null(markerInfo(object)$Gene))
          geneName <- as.character(markerInfo(object)[marker,"Gene"])
        else
          geneName <- "?"
        
        if(is.na(geneName) || geneName<" ")
          geneName <- "?"
        
        if(!is.null(markerInfo(object)$Position))
          pos <- as.character(markerInfo(object)[marker,"Position"])
        else
          pos <- "?"
        
        if(is.na(pos) || pos<" ")
          pos <- "?"
        
        retval[1,"Gene"] <- geneName
        retval[1,"Marker"] <- marker
        retval[1,"Position"] <- pos
        retval[names(count.na),"Allele"] <- sprintf("%s",names(count.na))
        retval[names(count.na),"Count"] <- sprintf("%6d",count.na)
        retval[names(freq),"Freq"] <- sprintf("%1.3f",freq)
        retval[names(ci.low),"CI-Lower"] <- sprintf("%1.3f",ci.low)
        retval[names(ci.high),"CI-Upper"] <- sprintf("%1.3f",ci.high)
        rownames(retval) <- rep("", nrow(retval))
        
        if(!missing(groupName) && groupName !="ALL")
          retval[-nrow(retval),"Group"] <- groupName
        else
          retval[-nrow(retval),"Group"] <- "ALL"
        
        tableList[[marker]] <- retval
      }

    return(tableList) ## return to markerLoop
  }

==> GeneticsBase_1.12.0.tar.gz: GeneticsBase/R/alleleSummary.R <==
## library(gdata)

alleleSummary <- function(object,
                          by=NULL,
                          confidence=0.95,
                          alpha=1-confidence,
                          show=TRUE,
                          verbose=FALSE,
                          includeOverall=FALSE,
                          omitRepeats=TRUE,
                          ...)
  {

    ## Ensure that all "Gene" names (if present) appear together in the markerInfo
    if(!is.null(markerInfo(object)$Gene))
      tGene <- markerInfo(object)$Gene
    else
      tGene <- rep(1, nrow(markerInfo(object)))

    if(!is.null(markerInfo(object)$Position))
      tPosition <- markerInfo(object)$Position
    else
      tPosition <- rep(1, nrow(markerInfo(object)))

    if(!is.null(markerInfo(object)$Name))
      tMarker <- markerInfo(object)$Name
    else
      tMarker <- rep(1, nrow(markerInfo(object)))

    ord <- order(tGene, tPosition, tMarker)
    markers <- markerInfo(object)$Name[ord]

    ## Ensure that if by is present then it matches one of the columns of sampleInfo
    if(!missing(by) && !(by %in% colnames(sampleInfo(object))))
      {
        paste("Warning! - by is not present in sampleInfo of the object",
              "Using entire dataset\n")
        by <- "ALL"
      }

    ## If the "by" info is not present in markerInfo, assign it to "ALL"
    if(missing(by))
      by <- "ALL"

    ## Saving sampleInfo as it is used in several places.
    samp.info <- sampleInfo(object)

    ##
    if(!is.null( markerInfo(object)$Gene ) )
       {
         geneList <- sort(unique(as.character( markerInfo(object)$Gene )))
         markerList <- split(markerInfo(object)$Name, markerInfo(object)$Gene)
       }
    else
      {
        geneList <- "ALL"
        markerList <- split(markerInfo(object)$Name, "ALL")
        ## markerList <- as.list(markerInfo(object)$Name)
        ## names(markerList) <- rep("ALL",length(markerList))
      }

    if(!missing(by) && (by %in% colnames(sampleInfo(object))))
      {
        groupList <- levels(as.factor(sampleInfo(object)[,by]))
        if(includeOverall)
          groupList = c("ALL",groupList)
      }
    else
      groupList <- "ALL"

    nMarkers <- nmarker(object)
    nLevels <- length(groupList)

    ##    totalNoAlleles <- sum(nallele(object)) + nmarker(object)
    ##    ## as we want to report the NA summary also


    ## inner calculation from markerLoop here
    totalNoAlleles <- 0
    for (marker in markers)
      {
        tt <- transTables(object,marker)
        na.i <- which(apply(tt[,-1, drop=FALSE],1,function(x)all(is.na(x))))
        na.allele <- ifelse(length(na.i), tt[na.i,1],"none")
        if(is.na(na.allele))na.allele <- "NA"
        aC <- alleleCount(object, marker=marker)
        count <-  apply(aC, 2, sum, na.rm=TRUE)
        names(count) <- unlist(alleleLevels(object, marker))
        if(na.allele!='none') count.na <- count
        else count.na <- count ##c(count, "NA"=0)
        totalNoAlleles <- totalNoAlleles + length(count)
      }
    
    columnNames <- c("Gene", "Marker", "Position", "Group", "Allele", 
                     "Count", "Freq", "CI-Lower", "CI-Upper")

    len.cols <- length(columnNames)
    resultTab <- matrix("", nrow=(totalNoAlleles + nMarkers)* nLevels   ,
                      ncol=len.cols)  # the +1*.. is for blank separator lines
    colnames(resultTab) <- columnNames

    if(by != "ALL")
    {
      factor.split <- as.factor(samp.info[,by])
      subGroups <- split(samp.info, factor.split)
    }
    else
      {
        subGroups <- split(samp.info, 1)
        names(subGroups) <- "ALL"
      }
        
    counter <- 1
    for(gene in geneList)
      {
        # add gene infomration to the table
        resultTab[counter, "Gene"] <- gene

        markers.gene <- markerList[[gene]]
        ## Marker Loop
        for(marker in markers.gene)
          {
            ##            if (by == "ALL")
            ##            object <- object
            ##        else {
            ##        object <- object[, as.numeric(rownames)]
            ##            }

            if(!omitRepeats)
              resultTab[counter, "Gene"] <- gene
            resultTab[counter, "Marker"] <- marker

            if(!is.null(markerInfo(object)$Position))
              resultTab[counter, "Position"] <-
                as.character(markerInfo(object)[marker,"Position"])
            else
              resultTab[counter, "Position"] <- "?"
            ## Inner most loop is starting here
            for(group in groupList)
              {
                resultTab[counter, "Group"] <- as.vector(groupList[group])

                ## inner calculation from markerLoop here
                ##aC <- alleleCount(object, marker=marker)

                curr.group <- group
                groupName <- group

                if(group=="ALL")
                  {
                    object.curr <- object
                  }
                else
                  {
                    subset.rows <- rownames(subGroups[[curr.group]]) %in% 
                                   rownames(sampleInfo(object))
                    object.curr <- object[,subset.rows]
                  }

                tt <- transTables(object,marker)
                na.i <- which(apply(tt[,-1,drop=FALSE],1,function(x)all(is.na(x))))
                na.allele <- ifelse(length(na.i), tt[na.i,1],"none")
                if(is.na(na.allele))na.allele <- "NA"
                aC <- alleleCount(object.curr, marker=marker)
                count <-  apply(aC, 2, sum, na.rm=TRUE)
                names(count) <- unlist(alleleLevels(object.curr, marker))
                if(na.allele!='none'){
                  na.i <- ifelse(na.allele=='NA',which(is.na(names(count))),
                                   which(names(count)==na.allele))
                  l <- 1:length(count)
                  l[na.i] <- length(count)
                  l[length(count)] <- na.i
                  count.na <- count[l]
                  freq  <- prop.table(count[-na.i])
                  n <- sum(count[-na.i])
                  count <- count[-na.i]
                }
                else{
                  count.na <- count ##c(count, "NA"=0)
                  freq  <- prop.table(count)
                  n <- sum(count)
                }
                
                ## FIXME: These calculations are based on binomial
                ## probabilities and may not be correct for the true
                ## multinomial distributions
                stderr <- sqrt(freq*(1-freq)/n)
                ci.low  <- qbinom(p=alpha/2, prob=freq, size=n)/n
                ci.high <- qbinom(p=1-alpha/2, prob=freq, size=n)/n

                count.na <- c(count.na, "") ## To incroporate the blank line
                len.count.na <- length(count.na)
                
                resultTab[(counter: (counter+len.count.na-1)), "Allele"] <- sprintf("%s",names(count.na))
                resultTab[(counter: (counter+len.count.na-1)), "Count"] <- sprintf("%6s",count.na)
                resultTab[(counter: (counter+length(freq)-1)), "Freq"] <- sprintf("%1.3f",freq)
                resultTab[(counter: (counter+length(ci.low)-1)), "CI-Lower"] <- sprintf("%1.3f",ci.low)
                resultTab[(counter: (counter+length(ci.high)-1)), "CI-Upper"] <- sprintf("%1.3f",ci.high)


                if(!missing(groupName) && groupName !="ALL")
                  resultTab[(counter: (counter+len.count.na-2)), "Group"] <- groupName
                else
                  resultTab[(counter: (counter+len.count.na-2)), "Group"] <- "ALL"

                if(verbose && counter%%50==0)
                  cat("Working on marker #", counter, "\n",sep="")
                
                counter <- counter+len.count.na
              }
          }
        
        }

    retval <- list()

    rownames(resultTab) <- rep("", nrow(resultTab))
    retval$data <- resultTab
    
    retval$footer <- paste("Confidence intervals width is 95%, computed using",
                           "the exact quantiles for the binomial distribution.")
    
    retval$call <- match.call()

    class(retval) <- c("alleleSummary","GeneticsBaseSummary")
    
    return(retval)
  }



markerLoop <- function(object,
                       groupName,
                       confidence=0.95,
                       alpha=1-confidence,
                       show=TRUE,
                       verbose=FALSE,
                       markers=markerNames(object),
                       ...)
  {

    ## Table columns: 
    ## One line per marker per allele per covariate
    columnNames <- c("Gene", "Marker", "Position", "Group", "Allele", 
                  "Count", "Freq", "CI-Lower", "CI-Upper")


    tableList <- vector(mode="list", length=nmarker(object))
    names(tableList) <- markerNames(object)

    ## Replacing this loop with vector operations (ie for stderr,
    ## ci.low, ci.high, ...) might speed it up considerably
    counter <- 1
    for(marker in markers)
      {
        if(verbose && counter%%50==0)
          cat("Working on marker #", counter, "\n",sep="")
        counter <- counter+1
        tt <- transTables(object,marker)
        na.i <- which(apply(tt[,-1,drop=FALSE],1,function(x)all(is.na(x))))
        na.allele <- ifelse(length(na.i), tt[na.i,1],"none")
        if(is.na(na.allele))na.allele <- "NA"
        aC <- alleleCount(object, marker=marker)

        count <- apply(
                       aC,
                       2,
                       sum,
                       na.rm=TRUE)
        names(count) <- unlist(alleleLevels(object, marker))
        if(na.allele!='none'){
          na.i <- ifelse(na.allele=='NA',which(is.na(names(count))),
                                   which(names(count)==na.allele))
          l <- 1:length(count)
          l[na.i] <- length(count)
          l[length(count)] <- na.i
          count.na <- count[l]
          freq  <- prop.table(count[-na.i])
          n <- sum(count[-na.i])
          count <- count[-na.i]
        }
        else{
          count.na <- count ##c(count, "NA"=0)
          freq  <- prop.table(count)
          n <- sum(count)
        }
        #count.na <- c(count, "NA"=sum(is.na(aC[,1])))
        
        #freq  <- prop.table(count)
        #n <- sum(count)
        
        ## FIXME: These calculations are based on binomial
        ## probabilities and may not be correct for the true
        ## multinomial distributions
        stderr <- sqrt(freq*(1-freq)/n)
        ci.low  <- qbinom(p=alpha/2, prob=freq, size=n)/n
        ci.high <- qbinom(p=1-alpha/2, prob=freq, size=n)/n
        
        retval <- matrix("", ncol=length(columnNames), nrow=length(count.na)+1)
        rownames(retval) <- c(names(count.na),"")
        colnames(retval) <- columnNames

        if(!is.null(markerInfo(object)$Gene))
          geneName <- as.character(markerInfo(object)[marker,"Gene"])
        else
          geneName <- "?"
        
        if(is.na(geneName) || geneName<" ")
          geneName <- "?"
        
        if(!is.null(markerInfo(object)$Position))
          pos <- as.character(markerInfo(object)[marker,"Position"])
        else
          pos <- "?"
        
        if(is.na(pos) || pos<" ")
          pos <- "?"
        
        retval[1,"Gene"] <- geneName
        retval[1,"Marker"] <- marker
        retval[1,"Position"] <- pos
        retval[names(count.na),"Allele"] <- sprintf("%s",names(count.na))
        retval[names(count.na),"Count"] <- sprintf("%6d",count.na)
        retval[names(freq),"Freq"] <- sprintf("%1.3f",freq)
        retval[names(ci.low),"CI-Lower"] <- sprintf("%1.3f",ci.low)
        retval[names(ci.high),"CI-Upper"] <- sprintf("%1.3f",ci.high)
        rownames(retval) <- rep("", nrow(retval))
        
        if(!missing(groupName) && groupName !="ALL")
          retval[-nrow(retval),"Group"] <- groupName
        else
          retval[-nrow(retval),"Group"] <- "ALL"
        
        tableList[[marker]] <- retval
      }

    return(tableList) ## return to markerLoop
  }
